@@ -1,16 +1,29 @@
 <?php
 /**
- * This class defines the Horde_Image:: API, and also provides some
- * utility functions, such as generating highlights of a color.
- *
- * Copyright 2002-2014 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2015 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
- * @author  Chuck Hagenbuch <chuck@horde.org>
- * @author  Michael J. Rubinsky <mrubinsk@horde.org>
- * @package Image
+ * @author    Chuck Hagenbuch <chuck@horde.org>
+ * @author    Michael J. Rubinsky <mrubinsk@horde.org>
+ * @author    Jan Schneider <jan@horde.org>
+ * @category  Horde
+ * @license   http://www.horde.org/licenses/lgpl21 LGPL-2.1
+ * @package   Image
+ */
+
+/**
+ * This class defines the Horde_Image API, and also provides some utility
+ * functions, such as generating highlights of a color.
+ *
+ * @author    Chuck Hagenbuch <chuck@horde.org>
+ * @author    Michael J. Rubinsky <mrubinsk@horde.org>
+ * @author    Jan Schneider <jan@horde.org>
+ * @category  Horde
+ * @copyright 2002-2015 Horde LLC
+ * @license   http://www.horde.org/licenses/lgpl21 LGPL-2.1
+ * @package   Image
  */
 abstract class Horde_Image_Base extends EmptyIterator
 {
@@ -22,12 +35,11 @@ abstract class Horde_Image_Base extends EmptyIterator
     protected $_background = 'white';
 
     /**
-     * Observers.
+     * Capabilites of this driver.
      *
      * @var array
      */
-    protected $_observers = array();
-
+    protected $_capabilities = array();
 
     /**
      * The current image data.
@@ -37,7 +49,7 @@ abstract class Horde_Image_Base extends EmptyIterator
     protected $_data = '';
 
     /**
-     * Logger
+     * Logger.
      */
     protected $_logger;
 
@@ -87,19 +99,18 @@ abstract class Horde_Image_Base extends EmptyIterator
      * Constructor.
      *
      * @param array $params   The image object parameters. Values include:
-     *<pre>
-     *   (optional)width  - The desired image width
-     *   (optional)height - The desired image height
-     *   (optional)type   - The image type (png, jpeg etc...) If not provided,
-     *                      or set by the setType method, any image output will
-     *                      be converted to the default image type of png.
-     *   (optional)data   - The image binary data.
-     *</pre>
-     * @param array $context  The object context - configuration, injected objects
-     *<pre>
-     *   (required)tmpdir - Temporary directory
-     *   (optional)logger - The logger
-     *</pre>
+     *                        - background: (string) The background color.
+     *                                      DEFAULT: white.
+     *                        - data: (string) The image binary data.
+     *                        - height: (integer) The desired image height.
+     *                        - type: (string) The output image type (png, jpeg
+     *                                etc.). DEFAULT: png.
+     *                        - width: (integer) The desired image width.
+     * @param array $context  The object context - configuration, injected
+     *                        objects:
+     *                        - logger: (Horde_Log_Logger) A logger.
+     *                        - tmpdir: [REQUIRED] (string) Temporary directory.
+     *
      * @throws InvalidArgumentException
      */
     protected function __construct($params, $context = array())
@@ -108,10 +119,15 @@ abstract class Horde_Image_Base extends EmptyIterator
         $this->_context = $context;
 
         if (empty($context['tmpdir'])) {
-            throw new InvalidArgumentException('A path to a temporary directory is required.');
+            throw new InvalidArgumentException(
+                'A path to a temporary directory is required.'
+            );
+        }
+        $this->_tmpdir = $context['tmpdir'];
+        if (!empty($context['logger'])) {
+            $this->_logger = $context['logger'];
         }
 
-        $this->_tmpdir = $context['tmpdir'];
         if (isset($params['width'])) {
             $this->_width = $params['width'];
         }
@@ -125,18 +141,23 @@ abstract class Horde_Image_Base extends EmptyIterator
             }
             $this->_type = $params['type'];
         }
-
-        if (!empty($context['logger'])) {
-            $this->_logger = $context['logger'];
+        if (!empty($params['background'])) {
+            $this->_background = $params['background'];
         }
-
-        $this->_background = isset($params['background']) ? $params['background'] : 'white';
     }
 
     /**
-     * Getter for the capabilities array
+     * Catch-all method so that we don't error out when calling an unsupported
+     * manipulation method.
+     */
+    public function __call($method, $args)
+    {
+    }
+
+    /**
+     * Returns the capabilities.
      *
-     * @return array
+     * @return array  A list of backend capabilities.
      */
     public function getCapabilities()
     {
@@ -144,11 +165,11 @@ abstract class Horde_Image_Base extends EmptyIterator
     }
 
     /**
-     * Check the existence of a particular capability.
+     * Checks the existence of a particular capability.
      *
      * @param string $capability  The capability to check for.
      *
-     * @return boolean
+     * @return boolean  True if the backend has this capability.
      */
     public function hasCapability($capability)
     {
@@ -156,7 +177,7 @@ abstract class Horde_Image_Base extends EmptyIterator
     }
 
     /**
-     * Generate image headers.
+     * Sends HTTP headers for the image.
      */
     public function headers()
     {
@@ -164,9 +185,9 @@ abstract class Horde_Image_Base extends EmptyIterator
     }
 
     /**
-     * Return the content type for this image.
+     * Returns the MIME type for this image.
      *
-     * @return string  The content type for this image.
+     * @return string  The MIME type for this image.
      */
     public function getContentType()
     {
@@ -174,9 +195,9 @@ abstract class Horde_Image_Base extends EmptyIterator
     }
 
     /**
-     * Getter for the simplified image type.
+     * Returns the image type.
      *
-     * @return string  The type of image (png, jpg, etc...)
+     * @return string  The type of this image (png, jpg, etc.).
      */
     public function getType()
     {
@@ -184,11 +205,11 @@ abstract class Horde_Image_Base extends EmptyIterator
     }
 
     /**
-     * Setter for the image type.
+     * Sets the output image type.
      *
-     * @param string $type  The simple type for the imag (png, jpg, etc...)
+     * @param string $type  An image type (png, jpg, etc.)
      *
-     * @return void
+     * @return string  The previous image type.
      */
     public function setType($type)
     {
@@ -203,8 +224,9 @@ abstract class Horde_Image_Base extends EmptyIterator
     }
 
     /**
-     * Draw a shaped point at the specified (x,y) point. Useful for
-     * scatter diagrams, debug points, etc. Draws squares, circles,
+     * Draws a shaped point at the specified (x,y) point.
+     *
+     * Useful for scatter diagrams, debug points, etc. Draws squares, circles,
      * diamonds, and triangles.
      *
      * @param integer $x     The x coordinate of the point to brush.
@@ -242,7 +264,7 @@ abstract class Horde_Image_Base extends EmptyIterator
     }
 
     /**
-     * Reset the image data to defaults.
+     * Resets the image data to defaults.
      */
     public function reset()
     {
@@ -253,7 +275,7 @@ abstract class Horde_Image_Base extends EmptyIterator
     }
 
     /**
-     * Get the height and width of the current image data.
+     * Returns the height and width of the current image data.
      *
      * @return array  An hash with 'width' containing the width,
      *                'height' containing the height of the image.
@@ -268,14 +290,12 @@ abstract class Horde_Image_Base extends EmptyIterator
             unlink($tmp);
         }
 
-        return array('width' => $this->_width,
-                     'height' => $this->_height);
+        return array('width' => $this->_width, 'height' => $this->_height);
     }
 
     /**
-     * Load the image data from a string.
+     * Loads the image data from a string.
      *
-     * @param string $id          An arbitrary id for the image.
      * @param string $image_data  The data to use for the image.
      */
     public function loadString($image_data)
@@ -285,50 +305,50 @@ abstract class Horde_Image_Base extends EmptyIterator
     }
 
     /**
-     * Load the image data from a file.
+     * Loads the image data from a file.
      *
      * @param string $filename  The full path and filename to the file to load
-     *                          the image data from. The filename will also be
-     *                          used for the image id.
+     *                          the image data from.
      *
-     * @return mixed  True if successful or already loaded, PEAR Error if file
-     *                does not exist or could not be loaded.
      * @throws Horde_Image_Exception
      */
     public function loadFile($filename)
     {
         $this->reset();
         if (!file_exists($filename)) {
-            throw new Horde_Image_Exception(sprintf("The image file, %s, does not exist.", $filename));
+            throw new Horde_Image_Exception(
+                sprintf("The image file, %s, does not exist.", $filename)
+            );
         }
         if (!$this->_data = file_get_contents($filename)) {
-            throw new Horde_Image_Exception(sprintf("Could not load the image file %s", $filename));
+            throw new Horde_Image_Exception(
+                sprintf("Could not load the image file %s", $filename)
+            );
         }
-
-        return true;
     }
 
     /**
-     * Ouputs image data to file.  If $data is false, outputs current
-     * image data after performing any pending operations on the data.
-     * If $data contains raw image data, outputs that data to file without
-     * regard for $this->_data
+     * Saves image data to file.
      *
-     * @param mixed  String of binary image data | false
+     * If $data is false, saves current image data after performing any pending
+     * operations on the data.  If $data contains raw image data, saves that
+     * data to file without regard for the current image data.
+     *
+     * @param string  String of binary image data.
      *
      * @return string  Path to temporary file.
      */
-    public function toFile($data = false)
+    public function toFile($data = null)
     {
         $tmp = Horde_Util::getTempFile('img', false, $this->_tmpdir);
-        $fp = @fopen($tmp, 'wb');
-        fwrite($fp, $data ? $data : $this->raw());
+        $fp = fopen($tmp, 'wb');
+        fwrite($fp, $data ?: $this->raw());
         fclose($fp);
         return $tmp;
     }
 
     /**
-     * Display the current image.
+     * Displays the current image.
      */
     public function display()
     {
@@ -353,10 +373,8 @@ abstract class Horde_Image_Base extends EmptyIterator
     /**
      * Attempts to apply requested effect to this image.
      *
-     * @param string $type    The type of effect to apply.
-     * @param array $params   Any parameters for the effect.
-     *
-     * @return boolean
+     * @param string $type   The type of effect to apply.
+     * @param array $params  Any parameters for the effect.
      */
     public function addEffect($type, $params)
     {
@@ -364,11 +382,11 @@ abstract class Horde_Image_Base extends EmptyIterator
         $params['logger'] = $this->_logger;
         $effect = Horde_Image_Effect::factory($type, $class, $params);
         $effect->setImageObject($this);
-        return $effect->apply();
+        $effect->apply();
     }
 
     /**
-     * Load a list of available effects for this driver.
+     * Returns a list of available effects for this driver.
      */
     public function getLoadedEffects()
     {
@@ -381,7 +399,9 @@ abstract class Horde_Image_Base extends EmptyIterator
                 if ($handle = opendir($path)) {
                     while (($file = readdir($handle)) !== false) {
                         if (substr($file, -4, 4) == '.php') {
-                            $this->_loadedEffects[] = substr($file, 0, strlen($file) - 4);
+                            $this->_loadedEffects[] = substr(
+                                $file, 0, strlen($file) - 4
+                            );
                         }
                     }
                 }
@@ -393,7 +413,9 @@ abstract class Horde_Image_Base extends EmptyIterator
                 if ($handle = opendir($path)) {
                     while (($file = readdir($handle)) !== false) {
                         if (substr($file, -4, 4) == '.php') {
-                            $this->_loadedEffects[] = substr($file, 0, strlen($file) - 4);
+                            $this->_loadedEffects[] = substr(
+                                $file, 0, strlen($file) - 4
+                            );
                         }
                     }
                 }
@@ -404,23 +426,28 @@ abstract class Horde_Image_Base extends EmptyIterator
     }
 
     /**
-     * Apply any effects in the effect queue.
+     * Applies any effects in the effect queue.
      */
     public function applyEffects()
     {
         $this->raw();
     }
 
+    /**
+     * Returns the current temporary directory.
+     *
+     * @return string  The current temporary directory.
+     */
     public function getTmpDir()
     {
         return $this->_tmpdir;
     }
 
     /**
-     * Utility function to zero out cached geometry information. Shouldn't
-     * really be called from client code, but is needed since Effects may need
-     * to clear these.
+     * Utility function to zero out cached geometry information.
      *
+     * Shouldn't really be called from client code, but is needed since effects
+     * may need to clear these.
      */
     public function clearGeometry()
     {
@@ -428,6 +455,11 @@ abstract class Horde_Image_Base extends EmptyIterator
         $this->_width = 0;
     }
 
+    /**
+     * Logs a message at debug level.
+     *
+     * @param string $message  The log message.
+     */
     protected function _logDebug($message)
     {
         if (!empty($this->_logger)) {
@@ -435,6 +467,11 @@ abstract class Horde_Image_Base extends EmptyIterator
         }
     }
 
+    /**
+     * Logs a message at error level.
+     *
+     * @param string $message  The log message.
+     */
     protected function _logErr($message)
     {
         if (!empty($this->_logger)) {
@@ -443,19 +480,29 @@ abstract class Horde_Image_Base extends EmptyIterator
     }
 
     /**
-     * Request a specific image from the collection of images.
+     * Returns a specific image from the pages of images.
      *
-     * @param integer $index  The index to return
+     * @param integer $index  The index to return.
      *
-     * @return Horde_Image_Base
+     * @return Horde_Image_Base  The requested image
      */
-    abstract function getImageAtIndex($index);
+    public function getImageAtIndex($index)
+    {
+        if ($index > 0) {
+            throw new Horde_Image_Exception('Image index out of bounds.');
+        }
+        $class = get_class($this);
+        return new $class(array('data' => $this->raw()), $this->_context);
+    }
 
     /**
-     * Return the number of image pages available in the image object.
+     * Returns the number of image pages available in the image object.
      *
-     * @return integer
+     * @return integer  The number of images.
      */
-    abstract function getImagePageCount();
+    public function getImagePageCount()
+    {
+        return 1;
+    }
 
 }
