@@ -1,27 +1,41 @@
 <?php
 /**
- * Image effect easily creating small, center-cropped thumbnails.
- * Requires IM version 6.3.8-3 or greater.
+ * Copyright 2002-2015 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
- * @author  Michael J. Rubinsky <mrubinsk@horde.org>
- * @package Image
+ * @author    Michael J. Rubinsky <mrubinsk@horde.org>
+ * @category  Horde
+ * @license   http://www.horde.org/licenses/lgpl21 LGPL-2.1
+ * @package   Image
+ */
+
+/**
+ * Image effect easily creating small, center-cropped thumbnails.
+ *
+ * Requires IM version 6.3.8-3 or greater.
+ *
+ * @author    Michael J. Rubinsky <mrubinsk@horde.org>
+ * @category  Horde
+ * @copyright 2002-2015 Horde LLC
+ * @license   http://www.horde.org/licenses/lgpl21 LGPL-2.1
+ * @package   Image
  */
 class Horde_Image_Effect_Im_CenterCrop extends Horde_Image_Effect
 {
     /**
      * Valid parameters:
-     *  <pre>
-     *    width    - Target width
-     *    height   - Target height
-     * </pre>
+     *   - width: (integer) Crop width.
+     *   - height: (integer Crop height.
      *
      * @var array
      */
     protected $_params = array();
 
+    /**
+     * Applies the effect.
+     */
     public function apply()
     {
         $this->_params = new Horde_Support_Array($this->_params);
@@ -34,8 +48,5 @@ class Horde_Image_Effect_Im_CenterCrop extends Horde_Image_Effect
         }
         $this->_image->addPostSrcOperation($command);
         $this->_image->clearGeometry();
-
-        return;
     }
-
-}
\ No newline at end of file
+}
