@@ -1,34 +1,47 @@
 <?php
 /**
- * Image effect for watermarking images with text for the im driver..
+ * Copyright 2007-2015 Horde LLC (http://www.horde.org/)
  *
- * Copyright 2007-2014 Horde LLC (http://www.horde.org/)
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
- * @author  Michael J. Rubinsky <mrubinsk@horde.org>
- * @package Image
+ * @author    Michael J. Rubinsky <mrubinsk@horde.org>
+ * @category  Horde
+ * @license   http://www.horde.org/licenses/lgpl21 LGPL-2.1
+ * @package   Image
+ */
+
+/**
+ * Image effect for watermarking images with text.
+ *
+ * @author    Michael J. Rubinsky <mrubinsk@horde.org>
+ * @category  Horde
+ * @copyright 2007-2015 Horde LLC
+ * @license   http://www.horde.org/licenses/lgpl21 LGPL-2.1
+ * @package   Image
  */
 class Horde_Image_Effect_Imagick_TextWatermark extends Horde_Image_Effect
 {
     /**
      * Valid parameters for watermark effects:
-     *
-     *   text (required)  - The text of the watermark.
-     *   halign           - The horizontal placement
-     *   valign           - The vertical placement
-     *   font             - The font name or family to use
-     *   fontsize         - The size of the font to use
-     *                      (small, medium, large, giant)
+     *   - text:     [REQUIRED] (string) The text of the watermark.
+     *   - halign:   (string) The horizontal placement
+     *   - valign:   (string) The vertical placement
+     *   - font:     (string) The font name or family to use
+     *   - fontsize: (string) The size of the font to use (small, medium,
+     *               large, giant)
      *
      * @var array
      */
-    protected $_params = array('halign' => 'right',
-                               'valign' => 'bottom',
-                               'font' => 'courier',
-                               'fontsize' => 'small');
+    protected $_params = array(
+        'halign'   => 'right',
+        'valign'   => 'bottom',
+        'font'     => 'courier',
+        'fontsize' => 'small'
+    );
 
     /**
-     * Add the watermark
-     *
+     * Applies the effect.
      */
     public function apply()
     {
@@ -63,11 +76,11 @@ class Horde_Image_Effect_Imagick_TextWatermark extends Horde_Image_Effect
         } else {
             $gravity = $v . $h;
         }
+
         /* Determine font point size */
         $point = Horde_Image::getFontSize($this->_params['fontsize']);
 
         //@TODO:
         throw new Horde_Image_Exception('Not Yet Implemented.');
     }
-
-}
\ No newline at end of file
+}
