@@ -1,28 +1,41 @@
 <?php
+/**
+ * Copyright 2003-2015 Horde LLC (http://www.horde.org/)
+ *
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.horde.org/licenses/lgpl21.
+ *
+ * @author    Chuck Hagenbuch <chuck@horde.org>
+ * @author    Michael J. Rubinsky <mrubinsk@horde.org>
+ * @category  Horde
+ * @license   http://www.horde.org/licenses/lgpl21 LGPL-2.1
+ * @package   Image
+ */
+
 /**
  * General class for fetching and parsing EXIF information from images.
  *
  * Works equally well with either the built in php exif functions (if PHP
- * compiled with exif support), the Exiftool package (more complete but slower),
- * or the bundled exif library.
+ * compiled with exif support), the Exiftool package (more complete but
+ * slower), or the bundled exif library.
  *
- * Copyright 2003-2014 Horde LLC (http://www.horde.org/)
- *
- * @author Michael J. Rubinsky <mrubinsk@horde.org>
- * @author Chuck Hagenbuch <chuck@horde.org>
- * @category Horde
- * @package Image
+ * @author    Chuck Hagenbuch <chuck@horde.org>
+ * @author    Michael J. Rubinsky <mrubinsk@horde.org>
+ * @category  Horde
+ * @copyright 2003-2015 Horde LLC
+ * @license   http://www.horde.org/licenses/lgpl21 LGPL-2.1
+ * @package   Image
  */
 class Horde_Image_Exif
 {
-    static protected $_titleFields = array(
+    protected static $_titleFields = array(
         'IPTC' => array('ObjectName'),
         'XMP'  => array('Title'),
         'EXIF' => array(),
         'COMPOSITE' => array()
     );
 
-    static protected $_descriptionFields = array(
+    protected static $_descriptionFields = array(
         'IPTC' => array('Caption-Abstract'),
         'XMP'  => array('Description'),
         'EXIF' => array('ImageDescription'),
@@ -37,7 +50,7 @@ class Horde_Image_Exif
      *
      * @return Horde_Image_Exif
      */
-    static public function factory($driver = null, $params = array())
+    public static function factory($driver = null, $params = array())
     {
         if (empty($driver) && function_exists('exif_read_data')) {
             $driver = 'Php';
@@ -53,32 +66,32 @@ class Horde_Image_Exif
     }
 
     /**
-     * Converts from Intel to Motorola endien.  Just reverses the bytes
-     * (assumes hex is passed in)
+     * Converts from Intel to Motorola endien.
      *
-     * @param $intel
+     * Just reverses the bytes (assumes hex is passed in)
      *
-     * @return
+     * @param string $intel
+     *
+     * @return string
      */
-    static public function intel2Moto($intel)
+    public static function intel2Moto($intel)
     {
         $len  = strlen($intel);
         $moto = '';
-        for($i = 0; $i <= $len; $i += 2) {
-            $moto .= substr($intel, $len-$i, 2);
+        for ($i = 0; $i <= $len; $i += 2) {
+            $moto .= substr($intel, $len - $i, 2);
         }
-
         return $moto;
     }
 
     /**
-     * Obtain an array of supported meta data fields.
+     * Obtains an array of supported meta data fields.
      *
      * @TODO: This should probably be extended by the subclass?
      *
      * @return array
      */
-    static public function getCategories()
+    public static function getCategories()
     {
         return array(
             'IPTC' => array(
@@ -149,7 +162,7 @@ class Horde_Image_Exif
     }
 
     /**
-     * Return a list of metadata fields that can by used for image titles.
+     * Returns a list of metadata fields that can by used for image titles.
      *
      * @param mixed $driver  A Horde_Image_Exif_Base instance or a string
      *                       specifying the driver in use.
@@ -157,7 +170,7 @@ class Horde_Image_Exif
      * @return array  An array of metadata field name hashes.
      * @since 2.1.0
      */
-    static public function getTitleFields($driver = null)
+    public static function getTitleFields($driver = null)
     {
         if (!is_null($driver) && is_array($driver)) {
             $driver = self::factory($driver[0], $driver[1]);
@@ -182,7 +195,8 @@ class Horde_Image_Exif
     }
 
     /**
-     * Return a list of metadata fields that can by used for image descriptions.
+     * Returns a list of metadata fields that can by used for image
+     * descriptions.
      *
      * @param mixed $driver  A Horde_Image_Exif_Base instance or a string
      *                       specifying the driver in use.
@@ -190,7 +204,7 @@ class Horde_Image_Exif
      * @return array  An array of metadata field hashes.
      * @since 2.1.0
      */
-    static public function getDescriptionFields($driver = null)
+    public static function getDescriptionFields($driver = null)
     {
         $map = self::getCategories();
         if (!is_null($driver) && is_array($driver)) {
@@ -216,15 +230,15 @@ class Horde_Image_Exif
     }
 
     /**
-     * Return a flattened array of supported metadata fields.
+     * Returns a flattened array of supported metadata fields.
      *
-     * @param mixed $driver  A Horde_Image_Exif_Base instance or a string
-     *                       specifying the driver in use.
+     * @param mixed $driver              A Horde_Image_Exif_Base instance or a
+     *                                   string specifying the driver in use.
      * @param boolean $description_only  Only return the field descriptions.
      *
      * @return array
      */
-    static public function getFields($driver = null, $description_only = false)
+    public static function getFields($driver = null, $description_only = false)
     {
         if (!is_null($driver) && is_array($driver)) {
             $driver = self::factory($driver[0], $driver[1]);
@@ -254,18 +268,25 @@ class Horde_Image_Exif
     /**
      * More human friendly exposure formatting.
      */
-    static protected function _formatExposure($data)
+    protected static function _formatExposure($data)
     {
         if ($data > 0) {
             if ($data > 1) {
-                return sprintf(Horde_Image_Translation::t("%d sec"), round($data, 2));
+                return sprintf(
+                    Horde_Image_Translation::t("%d sec"),
+                    round($data, 2)
+                );
             } else {
                 $n = $d = 0;
                 self::_convertToFraction($data, $n, $d);
                 if ($n <> 1) {
-                    return sprintf(Horde_Image_Translation::t("%4f sec"), $n / $d);
+                    return sprintf(
+                        Horde_Image_Translation::t("%4f sec"), $n / $d
+                    );
                 }
-                return sprintf(Horde_Image_Translation::t("%s / %s sec"), $n, $d);
+                return sprintf(
+                    Horde_Image_Translation::t("%s / %s sec"), $n, $d
+                );
             }
         } else {
             return Horde_Image_Translation::t("Bulb");
@@ -274,11 +295,12 @@ class Horde_Image_Exif
 
     /**
      * Converts a floating point number into a fraction.
+     *
      * Many thanks to Matthieu Froment for this code.
      *
      * (Ported from the Exifer library).
      */
-    static protected function _convertToFraction($v, &$n, &$d)
+    protected static function _convertToFraction($v, &$n, &$d)
     {
         $MaxTerms = 15;         // Limit to prevent infinite loop
         $MinDivisor = 0.000001; // Limit to prevent divide by zero
@@ -317,7 +339,8 @@ class Horde_Image_Exif
     }
 
     /**
-     * Convert an exif field into human-readable form.
+     * Converts an exif field into human-readable form.
+     *
      * Some of these cases are ported from the Exifer library, others were
      * changed from their implementation where the EXIF standard dictated
      * different behaviour.
@@ -327,7 +350,7 @@ class Horde_Image_Exif
      *
      * @return string  The converted data.
      */
-    static public function getHumanReadable($field, $data)
+    public static function getHumanReadable($field, $data)
     {
         switch ($field) {
         case 'ExposureMode':
