@@ -1,39 +1,63 @@
 <?php
 /**
- * This class implements the Horde_Image:: API for ImageMagick.
- *
- * Copyright 2002-2014 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2015 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
- * @author  Chuck Hagenbuch <chuck@horde.org>
- * @author  Mike Cochrane <mike@graftonhall.co.nz>
- * @author  Michael J. Rubinsky <mrubinsk@horde.org>
- * @package Image
+ * @author    Chuck Hagenbuch <chuck@horde.org>
+ * @author    Mike Cochrane <mike@graftonhall.co.nz>
+ * @author    Michael J. Rubinsky <mrubinsk@horde.org>
+ * @category  Horde
+ * @license   http://www.horde.org/licenses/lgpl21 LGPL-2.1
+ * @package   Image
+ */
+
+/**
+ * ImageMagick driver for the Horde_Image API.
+ *
+ * @author    Chuck Hagenbuch <chuck@horde.org>
+ * @author    Mike Cochrane <mike@graftonhall.co.nz>
+ * @author    Michael J. Rubinsky <mrubinsk@horde.org>
+ * @category  Horde
+ * @copyright 2002-2015 Horde LLC
+ * @license   http://www.horde.org/licenses/lgpl21 LGPL-2.1
+ * @package   Image
+ *
+ * @property-read  Imagick $imagick  The underlaying Imagick object.
  */
 class Horde_Image_Im extends Horde_Image_Base
 {
     /**
      * Capabilites of this driver.
      *
-     * @var array
+     * @var string[]
      */
     protected $_capabilities = array(
-        'resize',
-        'crop',
-        'rotate',
-        'grayscale',
-        'flip',
-        'mirror',
-        'sepia',
+        'arc',
         'canvas',
+        'circle',
+        'crop',
+        'dashedLine',
+        'flip',
+        'grayscale',
+        'line',
+        'mirror',
         'multipage',
-        'pdf');
+        'pdf',
+        'polygon',
+        'polyline',
+        'rectangle',
+        'resize',
+        'rotate',
+        'roundedRectangle',
+        'sepia',
+        'text',
+    );
 
     /**
-     * Operations to be performed before the source filename is specified on the
-     * command line.
+     * Operations to be performed before the source filename is specified on
+     * the command line.
      *
      * @var array
      */
@@ -49,36 +73,38 @@ class Horde_Image_Im extends Horde_Image_Base
 
     /**
      * An array of temporary filenames that need to be unlinked at the end of
-     * processing. Use addFileToClean() from client code (Effects) to add files
-     * to this array.
+     * processing.
+     *
+     * Use addFileToClean() from client code (effects) to add files to this
+     * array.
      *
      * @var array
      */
     protected $_toClean = array();
 
     /**
-     * Path to the convert binary
+     * Path to the convert binary.
      *
-     * @string
+     * @var string
      */
     protected $_convert = '';
 
     /**
-     * Path to the identify binary
+     * Path to the identify binary.
      *
-     * @string
+     * @var string
      */
     protected $_identify;
 
     /**
-     * Cache the number of image pages
+     * Cache for the number of image pages.
      *
      * @var integer
      */
     private $_pages;
 
     /**
-     * Track current page for the iterator
+     * The current page for the iterator.
      *
      * @var integer
      */
@@ -94,27 +120,33 @@ class Horde_Image_Im extends Horde_Image_Base
         parent::__construct($params, $context);
 
         if (empty($context['convert'])) {
-            throw new InvalidArgumentException('A path to the convert binary is required.');
+            throw new InvalidArgumentException(
+                'A path to the convert binary is required.'
+            );
         }
         $this->_convert = $context['convert'];
-
         if (!empty($context['identify'])) {
             $this->_identify = $context['identify'];
         }
+
         if (!empty($params['filename'])) {
             $this->loadFile($params['filename']);
         } elseif (!empty($params['data'])) {
             $this->loadString($params['data']);
         } else {
-            $cmd = "-size {$this->_width}x{$this->_height} xc:{$this->_background} +profile \"*\" {$this->_type}:__FILEOUT__";
+            $cmd = "-size {$this->_width}x{$this->_height} xc:{$this->_background} -strip {$this->_type}:__FILEOUT__";
             $this->executeConvertCmd($cmd);
         }
     }
 
     /**
-     * Publically visible raw method.
+     * Returns the raw data for this image.
      *
-     * @see self::_raw
+     * @param boolean $convert  If true, the image data will be returned in the
+     *                          target format, independently from any image
+     *                          operations.
+     *
+     * @return string  The raw image data.
      */
     public function raw($convert = false)
     {
@@ -124,11 +156,12 @@ class Horde_Image_Im extends Horde_Image_Base
     /**
      * Returns the raw data for this image.
      *
-     * @param boolean $convert  If true, the image data will be returned in the
-     *                          target format, even if no other image operations
-     *                          are present, otherwise, if no operations are
-     *                          present, the current raw data is returned
-     *                          unmodified.
+     * @param boolean $convert         If true, the image data will be returned
+     *                                 in the target format, independently from
+     *                                 any image operations.
+     * @param integer $index           An image index.
+     * @param boolean $preserver_data  Return the converted image but preserve
+     *                                 the current internal image data.
      *
      * @return string  The raw image data.
      */
@@ -144,32 +177,29 @@ class Horde_Image_Im extends Horde_Image_Base
         }
 
         $tmpin = $this->toFile($this->_data);
-
-        // Perform convert command if needed
-        if (count($this->_operations) || count($this->_postSrcOperations) || $convert) {
-            $tmpout = Horde_Util::getTempFile('img', false, $this->_tmpdir);
-            $command = $this->_convert . ' ' . implode(' ', $this->_operations)
-                . ' "' . $tmpin . '"\'[' . $index . ']\' '
-                . implode(' ', $this->_postSrcOperations)
-                . ' +profile "*" ' . $this->_type . ':"' . $tmpout . '" 2>&1';
-            $this->_logDebug(sprintf("convert command executed by Horde_Image_im::raw(): %s", $command));
-            exec($command, $output, $retval);
-            if ($retval) {
-                $error = sprintf("Error running command: %s", $command . "\n" . implode("\n", $output));
-                $this->_logErr($error);
-                throw new Horde_Image_Exception($error);
-            }
-
-            /* Empty the operations queue */
-            $this->_operations = array();
-            $this->_postSrcOperations = array();
-
-            /* Load the result */
-            $return = file_get_contents($tmpout);
-            if (!$preserve_data) {
-                $this->_data = $return;
-            }
+        $tmpout = Horde_Util::getTempFile('img', false, $this->_tmpdir);
+        $command = $this->_convert . ' ' . implode(' ', $this->_operations)
+            . ' "' . $tmpin . '"\'[' . $index . ']\' '
+            . implode(' ', $this->_postSrcOperations)
+            . ' -strip ' . $this->_type . ':"' . $tmpout . '" 2>&1';
+        $this->_logDebug(sprintf("convert command executed by Horde_Image_im::raw(): %s", $command));
+        exec($command, $output, $retval);
+        if ($retval) {
+            $error = sprintf("Error running command: %s", $command . "\n" . implode("\n", $output));
+            $this->_logErr($error);
+            throw new Horde_Image_Exception($error);
         }
+
+        /* Empty the operations queue */
+        $this->_operations = array();
+        $this->_postSrcOperations = array();
+
+        /* Load the result */
+        $return = file_get_contents($tmpout);
+        if (!$preserve_data) {
+            $this->_data = $return;
+        }
+
         @unlink($tmpin);
         @unlink($tmpout);
 
@@ -177,7 +207,7 @@ class Horde_Image_Im extends Horde_Image_Base
     }
 
     /**
-     * Reset the image data.
+     * Resets the image data.
      */
     public function reset()
     {
@@ -188,7 +218,7 @@ class Horde_Image_Im extends Horde_Image_Base
     }
 
     /**
-     * Resize the current image. This operation takes place immediately.
+     * Resizes the current image.
      *
      * @param integer $width        The new width.
      * @param integer $height       The new height.
@@ -201,25 +231,31 @@ class Horde_Image_Im extends Horde_Image_Base
         $resHeight = $height * 2;
         $this->_operations[] = "-size {$resWidth}x{$resHeight}";
         if ($ratio) {
-            $this->_postSrcOperations[] = (($keepProfile) ? "-resize" : "-thumbnail") . " {$width}x{$height}";
+            $this->_postSrcOperations[] =
+                ($keepProfile ? '-resize' : '-thumbnail')
+                . " {$width}x{$height}";
         } else {
-            $this->_postSrcOperations[] = (($keepProfile) ? "-resize" : "-thumbnail") . " {$width}x{$height}!";
+            $this->_postSrcOperations[] =
+                ($keepProfile ? '-resize' : '-thumbnail')
+                . " {$width}x{$height}!";
         }
-        // Reset the width and height instance variables since after resize
-        // we don't know the *exact* dimensions yet (especially if we maintained
-        // aspect ratio.
+
         // Refresh the data
         $this->raw();
+
+        // Reset the width and height instance variables since after resize we
+        // don't know the *exact* dimensions yet (especially if we maintained
+        // aspect ratio.
         $this->clearGeometry();
     }
 
     /**
-     * Crop the current image.
+     * Crops the current image.
      *
-     * @param integer $x1  x for the top left corner
-     * @param integer $y1  y for the top left corner
-     * @param integer $x2  x for the bottom right corner of the cropped image.
-     * @param integer $y2  y for the bottom right corner of the cropped image.
+     * @param integer $x1  x for the top left corner.
+     * @param integer $y1  y for the top left corner.
+     * @param integer $x2  x for the bottom right corner.
+     * @param integer $y2  y for the bottom right corner.
      */
     public function crop($x1, $y1, $x2, $y2)
     {
@@ -232,7 +268,7 @@ class Horde_Image_Im extends Horde_Image_Base
     }
 
     /**
-     * Rotate the current image. This is an atomic operation.
+     * Rotates the current image.
      *
      * @param integer $angle       The angle to rotate the image by,
      *                             in the clockwise direction.
@@ -249,7 +285,7 @@ class Horde_Image_Im extends Horde_Image_Base
     }
 
     /**
-     * Flip the current image.
+     * Flips the current image.
      */
     public function flip()
     {
@@ -257,7 +293,7 @@ class Horde_Image_Im extends Horde_Image_Base
     }
 
     /**
-     * Mirror the current image.
+     * Mirrors the current image.
      */
     public function mirror()
     {
@@ -265,7 +301,7 @@ class Horde_Image_Im extends Horde_Image_Base
     }
 
     /**
-     * Convert the current image to grayscale.
+     * Converts the current image to grayscale.
      */
     public function grayscale()
     {
@@ -273,39 +309,49 @@ class Horde_Image_Im extends Horde_Image_Base
     }
 
     /**
-     * Sepia filter.
+     * Applies a sepia filter.
      *
      * @param integer $threshold  Extent of sepia effect.
      */
-    public function sepia($threshold =  85)
+    public function sepia($threshold = 85)
     {
         $this->_operations[] = '-sepia-tone ' . $threshold . '%';
     }
 
     /**
-     * Draws a text string on the image in a specified location, with
-     * the specified style information.
+     * Draws a text string on the image in a specified location, with the
+     * specified style information.
      *
-     * @TODO: Need to differentiate between the stroke (border) and the fill color,
-     *        but this is a BC break, since we were just not providing a border.
+     * @TODO: Need to differentiate between the stroke (border) and the fill
+     *        color, but this is a BC break, since we were just not providing a
+     *        border.
      *
-     * @param string  $text       The text to draw.
-     * @param integer $x          The left x coordinate of the start of the text string.
-     * @param integer $y          The top y coordinate of the start of the text string.
-     * @param string  $font       The font identifier you want to use for the text.
-     * @param string  $color      The color that you want the text displayed in.
-     * @param integer $direction  An integer that specifies the orientation of the text.
-     * @param string  $fontsize   Size of the font (small, medium, large, giant)
+     * @param string $text        The text to draw.
+     * @param integer $x          The left x coordinate of the start of the
+     *                            text string.
+     * @param integer $y          The top y coordinate of the start of the text
+     *                            string.
+     * @param string $font        The font identifier you want to use for the
+     *                            text.
+     * @param string $color       The color that you want the text displayed in.
+     * @param integer $direction  An integer that specifies the orientation of
+     *                            the text.
+     * @param string $fontsize    Size of the font (small, medium, large, giant)
      */
-    public function text($string, $x, $y, $font = '', $color = 'black', $direction = 0, $fontsize = 'small')
+    public function text(
+        $string, $x, $y, $font = '', $color = 'black', $direction = 0,
+        $fontsize = 'small'
+    )
     {
         $string = addslashes('"' . $string . '"');
         $fontsize = Horde_Image::getFontSize($fontsize);
-        $this->_postSrcOperations[] = "-fill $color " . (!empty($font) ? "-font $font" : '') . " -pointsize $fontsize -gravity northwest -draw \"text $x,$y $string\" -fill none";
+        $this->_postSrcOperations[] = "-fill $color "
+            . (!empty($font) ? "-font $font" : '')
+            . " -pointsize $fontsize -gravity northwest -draw \"text $x,$y $string\" -fill none";
     }
 
     /**
-     * Draw a circle.
+     * Draws a circle.
      *
      * @param integer $x     The x coordinate of the centre.
      * @param integer $y     The y coordinate of the centre.
@@ -320,7 +366,7 @@ class Horde_Image_Im extends Horde_Image_Base
     }
 
     /**
-     * Draw a polygon based on a set of vertices.
+     * Draws a polygon based on a set of vertices.
      *
      * @param array $vertices  An array of x and y labeled arrays
      *                         (eg. $vertices[0]['x'], $vertices[0]['y'], ...).
@@ -337,7 +383,7 @@ class Horde_Image_Im extends Horde_Image_Base
     }
 
     /**
-     * Draw a rectangle.
+     * Draws a rectangle.
      *
      * @param integer $x       The left x-coordinate of the rectangle.
      * @param integer $y       The top y-coordinate of the rectangle.
@@ -354,7 +400,7 @@ class Horde_Image_Im extends Horde_Image_Base
     }
 
     /**
-     * Draw a rounded rectangle.
+     * Draws a rounded rectangle.
      *
      * @param integer $x       The left x-coordinate of the rectangle.
      * @param integer $y       The top y-coordinate of the rectangle.
@@ -364,7 +410,9 @@ class Horde_Image_Im extends Horde_Image_Base
      * @param string  $color   The line color of the rectangle.
      * @param string  $fill    The color to fill the rounded rectangle with.
      */
-    public function roundedRectangle($x, $y, $width, $height, $round, $color, $fill)
+    public function roundedRectangle(
+        $x, $y, $width, $height, $round, $color, $fill
+    )
     {
         $x1 = $x + $width;
         $y1 = $y + $height;
@@ -372,7 +420,7 @@ class Horde_Image_Im extends Horde_Image_Base
     }
 
     /**
-     * Draw a line.
+     * Draws a line.
      *
      * @param integer $x0     The x coordinate of the start.
      * @param integer $y0     The y coordinate of the start.
@@ -387,7 +435,7 @@ class Horde_Image_Im extends Horde_Image_Base
     }
 
     /**
-     * Draw a dashed line.
+     * Draws a dashed line.
      *
      * @param integer $x0           The x co-ordinate of the start.
      * @param integer $y0           The y co-ordinate of the start.
@@ -398,14 +446,17 @@ class Horde_Image_Im extends Horde_Image_Base
      * @param integer $dash_length  The length of a dash on the dashed line
      * @param integer $dash_space   The length of a space in the dashed line
      */
-    public function dashedLine($x0, $y0, $x1, $y1, $color = 'black', $width = 1, $dash_length = 2, $dash_space = 2)
+    public function dashedLine(
+        $x0, $y0, $x1, $y1, $color = 'black', $width = 1, $dash_length = 2,
+        $dash_space = 2
+    )
     {
        $this->_operations[] = "-stroke $color -strokewidth $width -draw \"line $x0,$y0 $x1,$y1\"";
     }
 
     /**
-     * Draw a polyline (a non-closed, non-filled polygon) based on a
-     * set of vertices.
+     * Draws a polyline (a non-closed, non-filled polygon) based on a set of
+     * vertices.
      *
      * @param array $vertices  An array of x and y labeled arrays
      *                         (eg. $vertices[0]['x'], $vertices[0]['y'], ...).
@@ -422,7 +473,7 @@ class Horde_Image_Im extends Horde_Image_Base
     }
 
     /**
-     * Draw an arc.
+     * Draws an arc.
      *
      * @param integer $x      The x coordinate of the centre.
      * @param integer $y      The y coordinate of the centre.
@@ -432,7 +483,9 @@ class Horde_Image_Im extends Horde_Image_Base
      * @param string  $color  The line color of the arc.
      * @param string  $fill   The fill color of the arc (defaults to none).
      */
-    public function arc($x, $y, $r, $start, $end, $color = 'black', $fill = 'none')
+    public function arc(
+        $x, $y, $r, $start, $end, $color = 'black', $fill = 'none'
+    )
     {
         // Split up arcs greater than 180 degrees into two pieces.
         $this->_postSrcOperations[] = "-stroke $color -fill $fill";
@@ -453,22 +506,23 @@ class Horde_Image_Im extends Horde_Image_Base
             list($x2, $y2) = Horde_Image::circlePoint($mid, $r * 2);
             list($x3, $y3) = Horde_Image::circlePoint($end, $r * 2);
 
-            // This seems to result in slightly better placement of
-            // pie slices.
-            $x++;
-            $y++;
-
-            $verts = array(array('x' => $x + $x3, 'y' => $y + $y3),
-                           array('x' => $x, 'y' => $y),
-                           array('x' => $x + $x1, 'y' => $y + $y1));
+            $verts = array(
+                array('x' => $x + round($x3), 'y' => $y + round($y3)),
+                array('x' => $x, 'y' => $y),
+                array('x' => $x + round($x1), 'y' => $y + round($y1))
+            );
 
             if ($mid > 90) {
-                $verts1 = array(array('x' => $x + $x2, 'y' => $y + $y2),
-                                array('x' => $x, 'y' => $y),
-                                array('x' => $x + $x1, 'y' => $y + $y1));
-                $verts2 = array(array('x' => $x + $x3, 'y' => $y + $y3),
-                                array('x' => $x, 'y' => $y),
-                                array('x' => $x + $x2, 'y' => $y + $y2));
+                $verts1 = array(
+                    array('x' => $x + round($x2), 'y' => $y + round($y2)),
+                    array('x' => $x, 'y' => $y),
+                    array('x' => $x + round($x1), 'y' => $y + round($y1))
+                );
+                $verts2 = array(
+                    array('x' => $x + round($x3), 'y' => $y + round($y3)),
+                    array('x' => $x, 'y' => $y),
+                    array('x' => $x + round($x2), 'y' => $y + round($y2))
+                );
 
                 $this->polygon($verts1, $fill, $fill);
                 $this->polygon($verts2, $fill, $fill);
@@ -482,6 +536,9 @@ class Horde_Image_Im extends Horde_Image_Base
         }
     }
 
+    /**
+     * Applies any effects in the effect queue.
+     */
     public function applyEffects()
     {
         $this->raw();
@@ -491,20 +548,19 @@ class Horde_Image_Im extends Horde_Image_Base
     }
 
     /**
-     * Method to execute a raw command directly in convert. Useful for executing
-     * more involved operations that may require multiple convert commands
-     * piped into each other for example. Really designed for use by im based
-     * Horde_Image_Effect objects..
+     * Method to execute a raw command directly in convert.
+     *
+     * Useful for executing more involved operations that may require multiple
+     * convert commands piped into each other for example. Really designed for
+     * use by Im based Horde_Image_Effect objects.
      *
      * The input and output files are quoted and substituted for __FILEIN__ and
-     * __FILEOUT__ respectfully. In order to support piped convert commands, the
-     * path to the convert command is substitued for __CONVERT__ (but the
+     * __FILEOUT__ respectfully. In order to support piped convert commands,
+     * the path to the convert command is substitued for __CONVERT__ (but the
      * initial convert command is added automatically).
      *
      * @param string $cmd    The command string, with substitutable tokens
      * @param array $values  Any values that should be substituted for tokens.
-     *
-     * @return
      */
     public function executeConvertCmd($cmd, $values = array())
     {
@@ -519,9 +575,11 @@ class Horde_Image_Im extends Horde_Image_Base
         $tmpout = Horde_Util::getTempFile('img', false, $this->_tmpdir);
 
         // Substitue them in the cmd string
-        $cmd = str_replace(array('__FILEIN__', '__FILEOUT__', '__CONVERT__'),
-                           array('"' . $tmpin . '"', '"' . $tmpout . '"', $this->_convert),
-                           $cmd);
+        $cmd = str_replace(
+            array('__FILEIN__', '__FILEOUT__', '__CONVERT__'),
+            array('"' . $tmpin . '"', '"' . $tmpout . '"', $this->_convert),
+            $cmd
+        );
 
         //TODO: See what else needs to be replaced.
         $cmd = $this->_convert . ' ' . $cmd . ' 2>&1';
@@ -539,10 +597,11 @@ class Horde_Image_Im extends Horde_Image_Base
     }
 
     /**
-     * Get the version of the convert command available. This needs to be
-     * publicly visable since it's used by various Effects.
+     * Returns the version of the convert command available.
      *
-     * @return A version string suitable for using in version_compare()
+     * This needs to be publicly visable since it's used by various effects.
+     *
+     * @return string  A version string suitable for using in version_compare().
      */
     public function getIMVersion()
     {
